/*
 * Copyright (c) 2004, Jeamy Lee, Austria.
 * JASCOM: Ascom for Java
 * Subproject of EOAUI (Extended Open Astronomical User Interface)
 * Version ALPHA 2
 * mailto:jascom.bugs@jeamy.com. All rights reserved.
 * Source and binaries could be found at
 * http://home.textdesign.com/jeamy/jascom/jascom.jar
 * For more information and/or support and bug reports/suggestions use the email above.
 *
 * JASCOM depends on the JACOB (JAva-COM Bridge) libraries
 * programmed by Dan Adler (Copyright (c) 1999) located at
 * http://danadler.com/jacob/
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Redistributions in any form must be accompanied by information on
 *    how to obtain complete source code for the JASCOM software.
 *
 * Redistribution of the JASCOM software is not permitted as part of any
 * commercial product that is targeted primarily at Java developers.
 * Such products include, but are not limited to: Java virtual machines,
 * integrated development environments, code libraries, and application
 * server products. Licensing terms for such distribution may be
 * obtained from the copyright holder.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package com.jeamy.jascom;

import com.jacob.com.*;
import com.jeamy.jascom.wrap.*;
import com.jeamy.jascom.exceptions.JascomException;

/**
 *
 * <p>Title: JASCOM: Ascom for Java</p>
 * <p>Description: Java to Ascom Wrapper</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: EOAUI</p>
 * @author Jeamy Lee
 * @version ALPHA 2
 */

// Available methods and properties are described in the
// ASCOM Telescope Interface Standard 2.0 Programmer's Reference
// Available at http://ascom-standards.org/index.html
// and in the ASCOM developer directory tree

public class DomeProperties extends Properties
{

  /**
   * DomeProperties
   */
  protected DomeProperties()
  {
	  super("DomeSim.Dome");
  }
  /**
   * DomeProperties
   * @param driverID String
   */
  public DomeProperties(String driverID)
  {
	  super(driverID);
  }

//  private void setCanSyncAzimut(boolean CanSyncAzimut)
//  {
//  }

  public boolean canSyncAzimut()
  {
	return comDispatcher.get("CanSyncAzimut").toBoolean();
  }

//  private void setCanSetAzimut(boolean CanSetAzimut)
//  {
//  }

  public boolean canSetAzimut()
  {
	return comDispatcher.get("CanSetAzimut").toBoolean();
  }

//  private void setCanSetShutter(boolean CanSetShutter)
//  {
//  }

  public boolean canSetShutter()
  {
	return comDispatcher.get("CanSetShutter").toBoolean();
  }

//  private void setCanSlave(boolean CanSlave)
//  {
//  }

  public boolean canSlave()
  {
	return comDispatcher.get("CanSlave").toBoolean();
  }

  public void setSlaved(boolean Slaved)
  {
	comDispatcher.put("Slaved", new Variant(Slaved));
  }

  public boolean slaved()
  {
	return comDispatcher.get("Slaved").toBoolean();
  }

//  private void canSetAltitude(boolean CanSetAzimut)
//  {
//  }

  public boolean canSetAltitude()
  {
	return comDispatcher.get("CanSetAltitude").toBoolean();
  }

//  private void shutterStatus(ShutterState shutterState)
//  {
//  }

  public ShutterState shutterStatus() throws JascomException
  {
	int e = comDispatcher.get("ShutterStatus").toInt();
	return ShutterState.getShutterState(e);
  }

  /**
   * close
   */
  public void close()
  {
	super.close();
  }
}
