/*
 * Copyright (c) 2004, Jeamy Lee, Austria.
 * JASCOM: Ascom for Java
 * Subproject of EOAUI (Extended Open Astronomical User Interface)
 * Version ALPHA 2
 * mailto:jascom.bugs@jeamy.com. All rights reserved.
 * Source and binaries could be found at
 * http://home.textdesign.com/jeamy/jascom/jascom.jar
 * For more information and/or support and bug reports/suggestions use the email above.
 *
 * JASCOM depends on the JACOB (JAva-COM Bridge) libraries
 * programmed by Dan Adler (Copyright (c) 1999) located at
 * http://danadler.com/jacob/
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Redistributions in any form must be accompanied by information on
 *    how to obtain complete source code for the JASCOM software.
 *
 * Redistribution of the JASCOM software is not permitted as part of any
 * commercial product that is targeted primarily at Java developers.
 * Such products include, but are not limited to: Java virtual machines,
 * integrated development environments, code libraries, and application
 * server products. Licensing terms for such distribution may be
 * obtained from the copyright holder.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package com.jeamy.jascom;

import com.jacob.activeX.*;
import com.jacob.com.*;
import com.jeamy.jascom.wrap.*;

/**
 *
 * <p>Title: JASCOM: Ascom for Java</p>
 * <p>Description: Java to Ascom Wrapper</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: EOAUI</p>
 * @author Jeamy Lee
 * @version ALPHA 2
 */

// Available methods and properties are described in the
// ASCOM Telescope Interface Standard 2.0 Programmer's Reference
// Available at http://ascom-standards.org/index.html
// and in the ASCOM developer directory tree

public class FocuserProperties
{
  protected String driverID = null;
  protected COMDispatcher comDispatcher = null;
  private ActiveXComponent focuser = null;

  /**
   * Properties
   */
  protected FocuserProperties()
  {
  }

  /**
   * Properties
   *
   * @param driverID String
   */
  public FocuserProperties(String driverID)
  {
	this.driverID = driverID;
	focuser = new ActiveXComponent(this.driverID);
	comDispatcher = new COMDispatcher(focuser);
  }

//  public void setTempCompAvailable(boolean TempCompAvailable)
//  {
//  }

  public boolean isTempCompAvailable()
  {
	return comDispatcher.get("TempCompAvailable").toBoolean();
  }

  public void setTempComp(boolean TempComp)
  {
	comDispatcher.put("TempComp", new Variant(TempComp));
  }

  public boolean tempComp()
  {
	return comDispatcher.get("TempComp").toBoolean();
  }

//  public void setTemperature(double Temperature)
//  {
//	}

  public double temperature()
  {
	return comDispatcher.get("Temperature").toDouble();
  }

//  private void setStepSize(double StepSize)
// {
//  }

  public double stepSize()
  {
	return comDispatcher.get("StepSize").toDouble();
  }

//  private void setPosition(long Position)
// {
//  }

  public long position()
  {
	return comDispatcher.get("Position").toInt();
  }

//  private void setMaxStep(long MaxStep)
//  {
//  }

  public long maxStep()
  {
	return comDispatcher.get("MaxStep").toInt();
  }

//  private void setMaxIncrement(long MaxIncrement)
//  {
//  }

  public long maxIncrement()
  {
	return comDispatcher.get("MaxIncrement").toInt();
  }

  public void setLink(boolean Link)
  {
	comDispatcher.put("Link", new Variant(Link));
  }

  public boolean link()
  {
	return comDispatcher.get("Link").toBoolean();
  }

//  private void setIsMoving(double IsMoving)
//  {
//  }

  public boolean isMoving()
  {
	return comDispatcher.get("IsMoving").toBoolean();
  }

//  private void setAbsolute(double Absolute)
//  {
//  }

  public boolean absolute()
  {
	return comDispatcher.get("Absolute").toBoolean();
  }

  /**
   * close
   */
  public void close()
  {
	focuser.release();
  }
}
