/*
 * Copyright (c) 2004, Jeamy Lee, Austria.
 * JASCOM: Ascom for Java
 * Subproject of EOAUI (Extended Open Astronomical User Interface)
 * Version ALPHA 2
 * mailto:jascom.bugs@jeamy.com. All rights reserved.
 * Source and binaries could be found at
 * http://home.textdesign.com/jeamy/jascom/jascom.jar
 * For more information and/or support and bug reports/suggestions use the email above.
 *
 * JASCOM depends on the JACOB (JAva-COM Bridge) libraries
 * programmed by Dan Adler (Copyright (c) 1999) located at
 * http://danadler.com/jacob/
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Redistributions in any form must be accompanied by information on
 *    how to obtain complete source code for the JASCOM software.
 *
 * Redistribution of the JASCOM software is not permitted as part of any
 * commercial product that is targeted primarily at Java developers.
 * Such products include, but are not limited to: Java virtual machines,
 * integrated development environments, code libraries, and application
 * server products. Licensing terms for such distribution may be
 * obtained from the copyright holder.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package com.jeamy.jascom;

import java.util.*;

import com.jacob.com.*;
import com.jeamy.jascom.helper.Date;
import com.jeamy.jascom.wrap.*;
import com.jeamy.jascom.exceptions.JascomException;

/**
 *
 * <p>Title: JASCOM: Ascom for Java</p>
 * <p>Description: Java to Ascom Wrapper</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: EOAUI</p>
 * @author Jeamy Lee
 * @version ALPHA 2
 */

// Available methods and properties are described in the
// ASCOM Telescope Interface Standard 2.0 Programmer's Reference
// Available at http://ascom-standards.org/index.html
// and in the ASCOM developer directory tree

public class TelescopeProperties extends Properties
{
  /**
   * TelescopeProperties
   */
  protected TelescopeProperties()
  {
	  super("ScopeSim.Telescope");
  }
  /**
   * TelescopeProperties
   * @param driverID String
   */
  public TelescopeProperties(String driverID)
  {
	  super(driverID);
  }

  public void setUTCDate(java.util.Date UTCDate)
  {
	Date d = new Date(UTCDate);
	comDispatcher.put("UTCDate", new Variant(d.getValue()));
  }

  public java.util.Date UTCDate()
  {
	return(new Date(comDispatcher.get("UTCDate").toDouble()).getDate());
  }

  public void setTrackingRates(List TrackingRates)
  {
	comDispatcher.put("TrackingRates", new Variant(TrackingRates));
  }

  public List trackingRates()
  {
	/** @todo utility methods for accessing collections */

	Object o = comDispatcher.get("TrackingRates");
	return null;
  }

  public void setTrackingRate(DriveRates TrackingRate)
  {
	comDispatcher.put("TrackingRate", new Variant(TrackingRate.enumb));
  }

  public DriveRates trackingRate() throws JascomException
  {
	int t = comDispatcher.get("TrackingRate").toInt();
	return DriveRates.getDriveRates(t);
  }

  public void setTracking(boolean Tracking)
  {
	comDispatcher.put("Tracking", new Variant(Tracking));
  }

  public boolean tracking()
  {
	return comDispatcher.get("Tracking").toBoolean();
  }

  public void setTargetRightAscension(double TargetRightAscension)
  {
	comDispatcher.put("TargetRightAscension", new Variant(TargetRightAscension));
  }

  public double targetRightAscension()
  {
	return comDispatcher.get("TargetRightAscension").toDouble();
  }

  public void setTargetDeclination(double TargetDeclination)
  {
	comDispatcher.put("TargetDeclination", new Variant(TargetDeclination));
  }

  public double targetDeclination()
  {
	return comDispatcher.get("TargetDeclination").toDouble();
  }

  public void setSlewSettleTime(int SlewSettleTime)
  {
	comDispatcher.put("SlewSettleTime", new Variant(SlewSettleTime));
  }

  public int slewSettleTime()
  {
	return comDispatcher.get("SlewSettleTime").toInt();
  }

  public void setSiteLongitude(double SiteLongitude)
  {
	comDispatcher.put("SiteLongitude", new Variant(SiteLongitude));
  }

  public double siteLongitude()
  {
	return comDispatcher.get("SiteLongitude").toDouble();
  }

  public void setSiteLatitude(double SiteLatitude)
  {
	comDispatcher.put("SiteLatitude", new Variant(SiteLatitude));
  }

  public double siteLatitude()
  {
	return comDispatcher.get("SiteLatitude").toDouble();
  }

  public void setSiteElevation(double SiteElevation)
  {
	comDispatcher.put("SiteElevation", new Variant(SiteElevation));
  }

  public double siteElevation()
  {
	return comDispatcher.get("SiteElevation").toDouble();
  }

//  private void setSiderealTime(double SiderealTime)
//  {
//  }

  public double siderealTime()
  {
	return comDispatcher.get("SiderealTime").toDouble();
  }

  public void setSideOfPier(PierSide SideOfPier)
  {
	System.out.println("SetSideOfPier:" + SideOfPier.enumb);
	comDispatcher.put("SideOfPier", new Variant(SideOfPier.enumb));
  }

  public PierSide sideOfPier() throws JascomException
  {
	int p = comDispatcher.get("SideOfPier").toInt();
	return PierSide.getPierSide(p);
  }

  public void setRightAscensionRate(double RightAscensionRate)
  {
	comDispatcher.put("RightAscensionRate", new Variant(RightAscensionRate));
  }

  public double rightAscensionRate()
  {
	return comDispatcher.get("RightAscensionRate").toDouble();
  }

//  private void setRightAscension(double RightAscension)
//  {
//  }

  public double rightAscension()
  {
	return comDispatcher.get("RightAscension").toDouble();
  }

//  private void setIsPulseGuiding(boolean IsPulseGuiding)
//  {
//  }

  public boolean isPulseGuiding()
  {
	return comDispatcher.get("IsPulseGuiding").toBoolean();
  }

  public void setGuideRateRightAscension(double GuideRateRightAscension)
  {
	comDispatcher.put("GuideRateRightAscension", new Variant(GuideRateRightAscension));
  }

  public double guideRateRightAscension()
  {
	return comDispatcher.get("GuideRateRightAscension").toDouble();
  }

  public void setGuideRateDeclination(double GuideRateDeclination)
  {
	comDispatcher.put("GuideRateDeclination", new Variant(GuideRateDeclination));

  }

  public double guideRateDeclination()
  {
	return comDispatcher.get("GuideRateDeclination").toDouble();
  }

//  private void setFocalLength(double FocalLength)
//  {
//  }

  public double focalLength()
  {
	return comDispatcher.get("FocalLength").toDouble();
  }

//  private void setEquatorialSystem(EquatorialCoordinateType EquatorialSystem)
//  {
//  }

  public EquatorialCoordinateType equatorialSystem() throws JascomException
  {
	int e = comDispatcher.get("EquatorialSystem").toInt();
	return EquatorialCoordinateType.getEquatorialCoordinateType(e);
  }

  public void setDoesRefraction(boolean DoesRefraction)
  {
	comDispatcher.put("DoesRefraction", new Variant(DoesRefraction));
  }

  public boolean doesRefraction()
  {
	return comDispatcher.get("DoesRefraction").toBoolean();
  }

  public void setDeclinationRate(double DeclinationRate)
  {
	comDispatcher.put("DeclinationRate", new Variant(DeclinationRate));
  }

  public double declinationRate()
  {
	return comDispatcher.get("DeclinationRate").toDouble();
  }

//  private void setDeclination(double Declination)
//  {
//  }

  public double declination()
  {
	return comDispatcher.get("Declination").toDouble();
  }

//  private void setCanUnpark(boolean canUnpark)
//  {
//  }

  public boolean canUnpark()
  {
	return comDispatcher.get("CanUnpark").toBoolean();
  }

//  private void setCanSyncAltAz(boolean canSyncAltAz)
//  {
//  }

  public boolean canSyncAltAz()
  {
	return comDispatcher.get("CanSyncAltAz").toBoolean();
  }

//  private void setCanSync(boolean canSync)
//  {
//  }

  public boolean canSync()
  {
	return comDispatcher.get("CanSync").toBoolean();
  }

//  private void setCanSlewAsync(boolean canSlewAsync)
//  {
//  }

  public boolean canSlewAsync()
  {
	return comDispatcher.get("CanSlewAsync").toBoolean();
  }

//  private void setCanSlewAltAzAsync(boolean canSlewAltAzAsync)
//  {
//  }

  public boolean canSlewAltAzAsync()
  {
	return comDispatcher.get("CanSlewAltAzAsync").toBoolean();
  }

//  private void setCanSlewAltAz(boolean canSlewAltAz)
//  {
//  }

  public boolean canSlewAltAz()
  {
	return comDispatcher.get("CanSlewAltAz").toBoolean();
  }

//  private void setCanSlew(boolean canSlew)
//  {
//  }

  public boolean canSlew()
  {
	return comDispatcher.get("CanSlew").toBoolean();
  }

//  private void setCanSetTracking(boolean canSetTracking)
//  {
//  }

  public boolean canSetTracking()
  {
	return comDispatcher.get("CanSetTracking").toBoolean();
  }

//  private void setCanSetRightAscensionRate(boolean canSetRightAscensionRate)
//  {
//  }

  public boolean canSetRightAscensionRate()
  {
	return comDispatcher.get("CanSetRightAscensionRate").toBoolean();
  }

//  private void setCanSetPierSide(boolean canSetPierSide)
//  {
//  }

  public boolean canSetPierSide()
  {
	return comDispatcher.get("CanSetPierSide").toBoolean();
  }

//  private void setCanSetGuideRates(boolean canSetGuideRates)
//  {
//  }

  public boolean canSetGuideRates()
  {
	return comDispatcher.get("CanSetGuideRates").toBoolean();
  }

//  private void setCanSetDeclinationRate(boolean canSetDeclinationRate)
//  {
//  }

  public boolean canSetDeclinationRate()
  {
	return comDispatcher.get("CanSetDeclinationRate").toBoolean();
  }

//  private void setCanPulseGuide(boolean canPulseGuide)
//  {
//  }

  public boolean canPulseGuide()
  {
	return comDispatcher.get("CanPulseGuide").toBoolean();
  }

//  private void setApertureDiameter(double ApertureDiameter)
//  {
//  }

  public double apertureDiameter()
  {
	return comDispatcher.get("ApertureDiameter").toDouble();
  }

//  private void setApertureArea(double ApertureArea)
//  {
//  }

  public double apertureArea()
  {
	return comDispatcher.get("ApertureArea").toDouble();
  }

//  private void setAlignmentMode(AlignmentModes alignmentMode)
//  {
//  }

  public AlignmentModes alignmentMode() throws JascomException
  {
	int a = comDispatcher.get("AlignmentMode").toInt();
	return AlignmentModes.getAlignmentModes(a);
  }

  /**
   * close
   */
  public void close()
  {
	super.close();
  }

}
