/*
 * Copyright (c) 2004 Jeamy Lee, Austria.
 * JASCOM: Ascom for Java
 * Subproject of EOAUI (Extended Open Astronomical User Interface)
 * Version ALPHA 2
 * mailto:jascom.bugs@jeamy.com. All rights reserved.
 * Source and binaries could be found at
 * http://home.textdesign.com/jeamy/jascom/jascom.jar
 * For more information and/or support and bug reports use above email.
 *
 * JASCOM depends on the JACOB (JAva-COM Bridge) libraries
 * programmed by Dan Adler (Copyright (c) 1999) located at
 * http://danadler.com/jacob/
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Redistributions in any form must be accompanied by information on
 *    how to obtain complete source code for the JASCOM software.
 *
 * Redistribution of the JASCOM software is not permitted as part of any
 * commercial product that is targeted primarily at Java developers.
 * Such products include, but are not limited to: Java virtual machines,
 * integrated development environments, code libraries, and application
 * server products. Licensing terms for such distribution may be
 * obtained from the copyright holder.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package com.jeamy.jascom.exceptions;

/**
 *
 * <p>Title: JASCOM: Ascom for Java</p>
 * <p>Description: Java to Ascom Wrapper</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: EOAUI</p>
 * @author Jeamy Lee
 * @version ALPHA 2
 */
public class JascomException extends Exception
{
  static final long serialVersionUID = 0;
  public static final String E_FIRST_S = "Schould be the first error message";
  public static final int E_FIRST = 0x80040400;
  public static final String E_ALIGN_NOT_SUPPORTED_S = "Alignmentmode not supported.";
  public static final int E_ALIGN_NOT_SUPPORTED = 0x80041500;
  public static final String E_DRIVER_RATE_NOT_SUPPORTED_S = "Driver rate not supported.";
  public static final int E_DRIVER_RATE_NOT_SUPPORTED = 0x80041501;
  public static final String E_COORD_NOT_SUPPORTED_S = "EQ coordinate type not supported.";
  public static final int E_COORD_NOT_SUPPORTED = 0x80041502;
  public static final String E_DIR_NOT_SUPPORTED_S = "Guide direction not supported.";
  public static final int E_DIR_NOT_SUPPORTED = 0x80041503;
  public static final String E_PIER_NOT_SUPPORTED_S = "Pier side not supported.";
  public static final int E_PIER_NOT_SUPPORTED = 0x80041504;
  public static final String E_RATE_NOT_SUPPORTED_S = "Rate not supported.";
  public static final int E_RATE_NOT_SUPPORTED = 0x80041505;
  public static final String E_SHUTTER_NOT_SUPPORTED_S = "Shutter state not supported.";
  public static final int E_SHUTTER_NOT_SUPPORTED = 0x80041506;
  public static final String E_AXIS_NOT_SUPPORTED_S = "Telescope axis not supported.";
  public static final int E_AXIS_NOT_SUPPORTED = 0x80041507;
  public static final String E_GUIDE_DIR_NOT_SUPPORTED_S = "Guide direction not supported.";
  public static final int E_GUIDE_DIR_NOT_SUPPORTED = 0x80041508;

  protected String error_string = "";
  protected Integer error = null;

  /**
   *
   */
  public JascomException()
  {
	super();
  }

  /**
   *
   * @param error int
   * @param error_string String
   */
  public JascomException(int error, String error_string)
  {
	super(error_string);

	this.error_string = error_string;
	this.error = new Integer(error);
  }

  /**
   *
   * @param jax JAscomException
   */
  public JascomException(JascomException jax)
  {
	super(jax.getErrorString());

	this.error_string = jax.getErrorString();
	this.error = new Integer(jax.getError());
  }

  /**
   *
   * @return String
   */
  public String getErrorString()
  {
	return error_string;
  }

  /**
   *
   * @return int
   */
  public int getError()
  {
	if(error != null)
	{
	  return error.intValue();
	} else
	{
	  return 0;
	}
  }
}
