/*
 * Copyright (c) 2004 Jeamy Lee, Austria.
 * JASCOM: Ascom for Java
 * Subproject of EOAUI (Extended Open Astronomical User Interface)
 * Version ALPHA 2
 * mailto:jascom.bugs@jeamy.com. All rights reserved.
 * Source and binaries could be found at
 * http://home.textdesign.com/jeamy/jascom/jascom.jar
 * For more information and/or support and bug reports/suggestions use the email above.
 *
 * JASCOM depends on the JACOB (JAva-COM Bridge) libraries
 * programmed by Dan Adler (Copyright (c) 1999) located at
 * http://danadler.com/jacob/
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Redistributions in any form must be accompanied by information on
 *    how to obtain complete source code for the JASCOM software.
 *
 * Redistribution of the JASCOM software is not permitted as part of any
 * commercial product that is targeted primarily at Java developers.
 * Such products include, but are not limited to: Java virtual machines,
 * integrated development environments, code libraries, and application
 * server products. Licensing terms for such distribution may be
 * obtained from the copyright holder.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package com.jeamy.jascom.test;

import com.jeamy.jascom.*;
import com.jeamy.jascom.helper.*;
import com.jeamy.jascom.wrap.*;

/**
 * Testclass
 *
 * <p>Title: JASCOM: Ascom for Java</p>
 * <p>Description: Java to Ascom Wrapper</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: EOAUI</p>
 * @author Jeamy Lee
 * @version ALPHA 2
 */

public class JAscomTest
{
    
  public static void main(String[] args)
  {
	String dflt_driver = "ScopeSim";
	
//	 Create Util object
	Util util = new Util();
	double d0 = 1.2345;
	String dms = util.degreesToDMS(d0,2);
	double d1 = util.DMSToDegrees(dms);
	String hms = util.degreesToHMS(d1,3);
	double h = util.HMSToHours(hms);
	if(dms.indexOf(',')>0||hms.indexOf(',')>0) {
		System.out.println("Warning! Wrong system localization.");
		System.out.println(d0 + " = " + d1 + " = "+ hms + " = " + h + "h = " + h*15. + "");
		System.out.println(	"Decimal '.' needs (not ',')! You maybe have problems with ASCOM Util object!");
	}
//	Create Chooser object and select/config driver
	Chooser choose = new Chooser();
//	String driverID = choose.choose();
//	 Or alternative, if we know the driver id, we could use:
	String driverID = choose.choose(dflt_driver + ".Telescope");
	if(driverID==null||driverID.length()==0)
		driverID = dflt_driver + ".Telescope";
	System.out.println("Driver: " + driverID);

//	  Initialize the Telescope object for the "Telescope Simulator Driver"
//	Telescope telescope = new Telescope("ScopeSim.Telescope");
	Telescope telescope = new Telescope(driverID);

//	  Available methods and properties are described in the
//	  ASCOM Telescope Interface Standard 2.0 Programmer's Reference
//	  Available at http://ascom-standards.org/index.html
//    and in the ASCOM developer directory tree
//
//    Quick and dirty installation instructions:

//    1. Download and install ASCOM Platform 3.0 Full Setup
//    located at http://ascom-standards.org/downloads.html

//    2. Download jacob.dll from http://danadler.com/jacob/ found in
//    JACOB binary distribution 1.7
//    There is a newer version located in the FILE-Section at
//    http://groups.yahoo.com/group/jacob-project/ (jacob_jre142fix_bin.zip)
//    This version could also be found on home.textdesign.com/jeamy/jascom/jacob_dll.jar
//    Add this file to your Path

//    3. Compile the files, start your Telescope Driver and connect it to the Telescope
//    or use the ASCOM Telescope simulator (prefered for testing).
//    Excecute com.jeamy.jascom.test.JAscomTest for some tests.
//    (Alternative you can start the binary version of JAscomTest with 'java -jar jascom.jar')
//
//    4. Create your Application using JAscom. That's all. Good luck! :)

	try
	{
//	  Test some methods

	  System.out.println("setupDialog() ");
	  telescope.setupDialog();

	  if(!telescope.connected()) {
		  System.out.println(telescope.name() + " Not Connected! ");
		  System.out.println("setConnected() ");
	      telescope.setConnected(true);
	  } else {
		  System.out.println(telescope.name() + " Connected...");
	  }

	  if(telescope.canUnpark()&& telescope.atPark()) {
		  System.out.println("\n atPark() = \"Yes\"");
		  System.out.println("\n unpark() ");
		  telescope.unpark();
		  Thread.sleep(3000L);
	  } else {
		  System.out.println("\n atPark() = \"No\"");
		  if (telescope.slewing()) {
			  System.out.println("\n slewing() = \"Yes\"");
			  System.out.println("\n abortSlew() ");
			  telescope.abortSlew();
			  Thread.sleep(3000L);
		  } else
			  System.out.println("\n slewing() = \"No\"");

	  }
	  if(telescope.canFindHome()&& !telescope.atHome()) {
		  System.out.println("\n atHome() = \"No\""); 
		  System.out.println("\n findHome() ");
		  telescope.findHome();
		  Thread.sleep(10000L);
	  } else
		  System.out.println("\n atHome() = \"Yes\"");
	  
	  if(telescope.alignmentMode()==AlignmentModes.algGermanPolar) {
		  System.out.println("\n sideOfPier()="+telescope.sideOfPier().toString());
		  System.out.println("\n destinationSideOfPier() ");
		  telescope.destinationSideOfPier(5.0, 30.0);
		  Thread.sleep(10000L);
	  }
	  TelescopeAxes axis = TelescopeAxes.axisPrimary;
	  System.out.println("\n canMoveAxis() ");
	  telescope.canMoveAxis(axis);
	  System.out.println("\n moveAxis() ");
	  telescope.moveAxis(axis, 0.5);
	  Thread.sleep(5000L);

	  System.out.println("\n pulseGuide() ");
	  boolean prop = telescope.canPulseGuide();
	  System.out.println("\n Property CanPulseGuide: " + (prop ? "Yes" : "No"));
	  GuideDirections direction = GuideDirections.guideEast;
	  telescope.pulseGuide(direction, 5000L);

	  if(telescope.canSetTracking() && telescope.tracking())
		  telescope.setTracking(false);
	  prop = telescope.canSlewAltAz();
	  System.out.println("\n Property CanSlewAltAz: " + (prop ? "Yes" : "No"));
	  System.out.println("\n slewToAltAz() ");
	  telescope.slewToAltAz(25.0, 60.0);
	  Thread.sleep(10000L);

	  prop = telescope.canSlewAltAzAsync();
	  System.out.println("\n Property canSlewAltAzAsync: " + (prop ? "Yes" : "No"));
	  System.out.println("\n slewToAltAzAsync() ");
	  telescope.slewToAltAzAsync(60.0, 30.0);
	  Thread.sleep(10000L);

	  if(telescope.canSetTracking() && !telescope.tracking())
		  telescope.setTracking(true);
	  prop = telescope.canSlew();
	  System.out.println("\n Property canSlew: " + (prop ? "Yes" : "No"));
	  double ra = util.HMSToHours("04:26:38.3");
	  double dec = util.DMSToDegrees("+12:05:06.9");
	  System.out.println("RA=" + ra);
	  System.out.println("DEC=" + dec);
	  System.out.println("\n slewToCoordinates() ");
	  telescope.slewToCoordinates(ra, dec);
	  Thread.sleep(10000L);

	  prop = telescope.canSlewAsync();
	  System.out.println("\n Property canSlewAsync: " + (prop ? "Yes" : "No"));
	  ra = util.HMSToHours("06:29:38.3");
	  dec = util.DMSToDegrees("+08:05:06.9");
	  System.out.println("RA=" + ra);
	  System.out.println("DEC=" + dec);
	  System.out.println("\n slewToCoordinatesAsync() ");
	  telescope.slewToCoordinatesAsync(ra, dec);
	  Thread.sleep(10000L);

	  System.out.println("\n slewToTarget() ");
	  telescope.setTargetDeclination(80.0);
	  telescope.setTargetRightAscension(10.0);
	  telescope.slewToTarget();
	  Thread.sleep(10000L);

	  System.out.println("\n slewToTargetAsync() ");
	  telescope.setTargetDeclination(40.0);
	  telescope.setTargetRightAscension(9.0);
	  telescope.slewToTargetAsync();
	  Thread.sleep(10000L);

	  if(telescope.canSetTracking() && telescope.tracking())
		  telescope.setTracking(false);
	  prop = telescope.canSyncAltAz();
	  System.out.println("\n Property canSyncAltAz: " + (prop ? "Yes" : "No"));
	  System.out.println("\n syncToAltAz() ");
	  telescope.syncToAltAz(10.0, 20.0);
	  Thread.sleep(10000L);

	  if(telescope.canSetTracking() && !telescope.tracking())
		  telescope.setTracking(true);
	  prop = telescope.canSync();
	  System.out.println("\n Property canSync: " + (prop ? "Yes" : "No"));
	  System.out.println("\n syncToCoordinates() ");
	  telescope.syncToCoordinates(2.5, 60.0);
	  Thread.sleep(10000L);

	  System.out.println("\n syncToTarget() ");
	  telescope.setTargetDeclination(20.0);
	  telescope.setTargetRightAscension(11.0);
	  telescope.syncToTarget();
	  Thread.sleep(1000L);

	  if(telescope.canSetPark()) {
		  System.out.println("\n setPark() ");
		  telescope.setPark();
		  Thread.sleep(1000L);
	  }
	  if(telescope.canPark()) {
		  System.out.println("\n park() ");
		  telescope.park();
	  }
	  // Test some methods in Profile object
	  // USE THE Profile CLASS WITH CAUTION. IT IS NOT TESTED VERY WELL! PRE ALPHA STATE!.
	  {
		// Create Profile object
		Profile profile = new Profile();
		System.out.println("Device type: " + profile.getDeviceType());
		String driver = "ScopeSim.Telescope";
		String name = "JASCOM Test";

		System.out.println("Keys: " + profile.subKeys(driver));
		System.out.println("Values: " + profile.values(driver));
		System.out.println("isRegistered: " + (profile.isRegistered(driver) ? "Yes" : "NO"));
		profile.writeValue(driver, name, "true");
		System.out.println("Value: " + profile.getValue(driver, name));
		profile.deleteValue(driver, name);
		System.out.println("Value after deletion: " + profile.getValue(driver, name));
		profile.close();
	  }

	} catch(Exception e)
	{
	  e.printStackTrace();
	} finally
	{
	  //release COM objects
	  telescope.close();
	  choose.close();
	  util.close();
	}
  }
}
